<?php
function fbStatusUpdater($post_ID, $cron = false) {
	global $fbStatusCookieFile, $fbDebugLog;
	$startTime = getMicrotime();

	$statusUpdate = get_option('fb-status-update');

	if ($statusUpdate === false) {
		// no settings stored
		return;
	} else {
		// the plugin has not been activated yet

		if (!isSet($statusUpdate["version"])) {
			return;
		}

		if (isSet($statusUpdate["fb-post-ids"]) && $statusUpdate["fb-post-ids"] != "" && $statusUpdate["fb-post-ids"] != "#") {
			// check if the current post id has already been sent to facebook
			if (strpos($statusUpdate["fb-post-ids"], "#".$post_ID."#") !== false) {
				// the post was already sent to facebook, do not lose time here
				return;
			}
		} else {
			// should already be set when installing the plugin from version 1.2
			// #1#2#3#100#1850#
			$statusUpdate["fb-post-ids"] = "#";
		}

		$fbUser = null;
		$fbPassword = null;
		$fbDebug = false;
		$fbWallId = null;
		$fbWallId2 = null;
		$fbLogEmail = null;
		$fbPostToProfile = true;
		$twUser = null;
		$twPassword = null;
		$fbLongUrl = true;
		$fbLogMessage = "";
		$fbAdvancedStatus = false;
		$jmpApiLogin = null;
		$jmpApiKey = null;
		$defaultStatusTemplate = "%POST-TITLE% %POST-URL%";

		if (isSet($statusUpdate["fb-email"]) && isSet($statusUpdate["fb-password"])) {
			$fbUser = $statusUpdate["fb-email"];
			$fbPassword = $statusUpdate["fb-password"];
		}

		if (isSet($statusUpdate["fb-debug"])) {
			$fbDebug = $statusUpdate["fb-debug"];
		}
		if (isSet($statusUpdate["fb-wall-id"])) {
			$fbWallId = $statusUpdate["fb-wall-id"];
		}
		if (isSet($statusUpdate["fb-wall-id-2"])) {
			$fbWallId2 = $statusUpdate["fb-wall-id-2"];
		}
		if (isSet($statusUpdate["fb-log-email"])) {
			$fbLogEmail = $statusUpdate["fb-log-email"];
		}
		if (isSet($statusUpdate["fb-post-to-profile"])) {
			$fbPostToProfile = $statusUpdate["fb-post-to-profile"];
		}
		if (isSet($statusUpdate["tw-user"]) && isSet($statusUpdate["tw-password"])) {
			$twUser = $statusUpdate["tw-user"];
			$twPassword = $statusUpdate["tw-password"];
		}
		if (isSet($statusUpdate["fb-long-url"])) {
			$fbLongUrl = $statusUpdate["fb-long-url"];
		}
		if (isSet($statusUpdate["fb-advanced-status"])) {
			$fbAdvancedStatus = $statusUpdate["fb-advanced-status"];
		}
		if (isSet($statusUpdate["jmp-api-login"])) {
			$jmpApiLogin = $statusUpdate["jmp-api-login"];
		}
		if (isSet($statusUpdate["jmp-api-key"])) {
			$jmpApiKey = $statusUpdate["jmp-api-key"];
		}
		if (isSet($statusUpdate["default-status-template"])) {
			$defaultStatusTemplate = $statusUpdate["default-status-template"];
		}

		if ($twUser == null && $twPassword == null && $fbUser == null && $fbPassword == null) {
			// nothing stored, just leave
			return;
		}
	}

	if (!is_writable($fbStatusCookieFile)) {
		if (!$cron) {
			echo("Your post has been published but there was an error while updating the Facebook status. The file ".$fbStatusCookieFile." is not writable from PHP. Please ensure PHP has the correct permissions set to write and update that file. If you don't know what I'm speaking about, please contact your server admin / webmaster. If you don't want to see this message every time you publish a new post while you try solving the problem, just <a href=\"/wp-admin/plugins.php\">disable this plugin</a>. <a href=\"http://codex.wordpress.org/Changing_File_Permissions\">More about file permissions on Wordpress</a>");
			exit();
		} else {
			// send email?
			if ($fbLogEmail != null) {
				sendLogEmail($fbLogEmail, "The Facebook Status Updater Cron tried to post an article on Facebook but the file ".$fbStatusCookieFile." is not writable from PHP.\n\n Please ensure PHP has the correct permissions set to write and update that file. If you don't know what I'm speaking about, please contact your server admin / webmaster. If you don't want to see this message every time you publish a new post while you try solving the problem, just disable this plugin.");
			}
			return;
		}
	}

	// post data
	$post = get_post($post_ID);
	if ($fbLongUrl) {
		$postUrl = get_permalink($post_ID);
	} else {
		$postUrl = get_bloginfo("wpurl")."/?p=".$post_ID;
	}
	$postTitle = $post->post_title;
	$postStatus = $post->post_status;
	unset($post);

	// if the post is schedule, do nothing
	if (($postStatus == "future" || $postStatus == "draft" || $postStatus == "private") && !$cron) {
		if ($fbLogEmail != null) {
			// if it is now a cron, warn the user that the post he scheduled will be published in the future
			sendLogEmail($fbLogEmail, "The artile \"".$postTitle."\" has not been shared on Facebook because is scheduled in the future. The plugin cron job will take care about it when it's time :) \n");
		}
		return;
	}

	$statusFacebook = $postTitle;
	$statusTwitter = $postTitle;

	if ($fbAdvancedStatus) {
		$customFbStatus = get_post_meta($post_ID, "fb-status-updater-status", true);
		$customTwStatus = get_post_meta($post_ID, "fb-status-updater-tw-status", true);
		$customSendToFacebook = get_post_meta($post_ID, "fb-status-updater-send-fb", true);
		$customSendToTwitter = get_post_meta($post_ID, "fb-status-updater-send-tw", true);
		$toSend = get_post_meta($post_ID, "fb-status-updater-send", true);

		// if the user chose not to send the current post, just add its id to the sent list and end
		if ($toSend == "false") {
			$statusUpdate["fb-post-ids"] .= $post_ID."#";
			update_option('fb-status-update', $statusUpdate);
			//echo("Not sending ".$post_ID." <br />"); exit();//debug
			return;
		}

		if (trim($customFbStatus."") != "") {
			$statusFacebook = $customFbStatus;
			//echo("Custom Facebook status: ".$statusFacebook."<br />"); //debug
		}

		if ($customSendToFacebook == "false") {
			$statusFacebook = null;
			//echo("The user decided not to send this post to Facebook<br />"); //debug
		}

		if (trim($customTwStatus."") != "") {
			$statusTwitter = $customTwStatus;
			//echo("Custom Twitter status: ".$customTwStatus."<br />"); //debug
		}

		if ($customSendToTwitter == "false") {
			$statusTwitter = null;
			//echo("The user decided not to send this post to Twitter<br />"); //debug
		}
	}

	if ($statusFacebook !== null) {
		// Facebook don't like shortened links, it asks for captcha. No good...
		$statusFacebook = str_replace("%POST-TITLE%", $statusFacebook, $defaultStatusTemplate);
		$statusFacebook = str_replace("%POST-URL%", $postUrl, $statusFacebook);
	}
	//echo("Final facebook status: [".$statusFacebook."]<br />"); //debug

	if ($statusTwitter !== null) {
		// url shortener, requires json
		if ($jmpApiLogin != null && $jmpApiKey !== null && function_exists("json_decode")) {

			$jMpUrl = "http://api.j.mp/shorten?version=2.0.1&longUrl=".urlencode($postUrl)."&login=".$jmpApiLogin."&apiKey=".$jmpApiKey."&history=1";

			$jMpResponse = getPage($jMpUrl);

			if ($jMpResponse === false || $jMpResponse == null || $jMpResponse == "") {

			} else {

				$jsonResult = json_decode($jMpResponse);

				if (isSet($jsonResult->statusCode) && strToLower($jsonResult->statusCode) == "ok") {
					if (isSet($jsonResult->results) && count($jsonResult->results) > 0) {
						foreach ($jsonResult->results as $result) { // only one time..
							$postUrl = $result->shortUrl;
						}
					}
				}
			}
		} else {
			$isGdApiUrl = "http://is.gd/api.php?longurl=".urlencode($postUrl);
			$shortUrl = getPage($isGdApiUrl);

			if (strpos($shortUrl, "Error: ") === false) {
				$postUrl = $shortUrl;
			}
		}

		//echo("Twitter short url: ".$postUrl."<br />"); //debug
	
		$tmpTwitterStatus =  str_replace("%POST-URL%", $postUrl, $defaultStatusTemplate);
		$availableTwitterStatusLength = 140 - strlen($tmpTwitterStatus) + strlen("%POST-TITLE%");

		if (strlen($statusTwitter) > $availableTwitterStatusLength) {
			$statusTwitter = substr($statusTwitter, 0, ($availableTwitterStatusLength - 3))."...";
		}

		$statusTwitter = str_replace("%POST-TITLE%", $statusTwitter, $tmpTwitterStatus);

		//echo("Final twitter status: [".$statusTwitter."]<br />"); //debug
	}

	if ($cron) {
		$fbLogMessage .= "the cron job of the Fb Status Updater plugin attempted to share a scheduled article on Facebook.\n\n";
	} else {
		$fbLogMessage .= "a new post has been published from your blog and the Fb Status Updater plugin attempted to share it on Facebook.\n\n";
	}

	// if FB email and password are not null, then login to facebook and send
	if ($fbUser != null && $fbPassword != null && $statusFacebook !== null) {
		$loginUrl = "https://login.facebook.com/login.php?m&locale=en_US&next=http://m.facebook.com/home.php%3Flocale=en_US";
		$postData = "locale=en_US&email=".$fbUser."&pass=".$fbPassword."&persistent=1&login=".urlencode("Log In");

		// clean the session data before starting
		// the file should already be clean but... you never know if someone else touches it with some malicious plugin
		deleteFbSessionData();

		$loginResponse = getPage($loginUrl, $postData);
		fbDebug("Login", $loginResponse, $fbDebug);

		// if the redirect can't be followed due to php restrictions, add a second call to the FB home
		if(ini_get('safe_mode') || ini_get("open_basedir")) {
			$homeUrl = "http://m.facebook.com/home.php?locale=en_US";
			$loginResponse = getPage($homeUrl);
			fbDebug("User home", $loginResponse, $fbDebug);
		}

		if (strpos($loginResponse, "type=\"password\"") !== false) {
			// login fallito
			if (!$cron) {
				// the link to the setting page might be wrong here... remember to get the wp root one day
				echo("Your post has been published but Facebook login failed. <a href=\"/wp-admin/edit.php\">Go to your posts</a> or <a href=\"/wp-admin/options-general.php?page=fb-status-updater.php\">check your Facebook login data</a>");
				deleteFbSessionData();
				exit();
			} else {
				// send an email?
				sendLogEmail($fbLogEmail, "The \"".$postTitle."\" post has been published and is online, so don't worry. The Facebook Status Updater plugin encountered an error while authenticating into Facebook. Please check your email and password in the Plugin Settings page");
				deleteFbSessionData();
				return;
			}

		}

		// if the user want to post on the main profile
		if ($fbPostToProfile) {

			preg_match('/name="post_form_id" value="(.*?)"/i', $loginResponse, $postFormId);

			if (isSet($postFormId[1])) {

				$postData = "post_form_id=".$postFormId[1]."&status=".urlencode($statusFacebook)."&update=Update+Status&locale=en_US";
				$statusFormUrl = "http://m.facebook.com/a/home.php?locale=en_US";

				$updateResponse = getPage($statusFormUrl, $postData);

				fbDebug("Status update", $updateResponse, $fbDebug);

				// somehow the page doesn't load in the locale set in the url, so it can't really be checked if the status was posted
				/*
				if (strpos(strToLower($updateResponse), "your status has been updated") === false) {
					echo("Your Facebook status has not been updated. The login succeed but then there has been a problem for an unknown reason. Try update your status manually and see what happens:<br /><br />".$title."<br /><br /><a href=\"/wp-admin/edit.php\">Go to your posts</a>.");
					// logout from facebook
					getPage("http://m.facebook.com/logout.php");
					deleteFbSessionData();
					exit();
				}
				*/

				$fbLogMessage .= "The status \"".$statusFacebook."\" has been published to the profile ".$fbUser."\n\n";
			}
		} else {
			// the user chose not to update the main profile status

		}

		// if wallId, try post to the wanted wall
		if ($fbWallId != NULL && $fbWallId != "") {
			$wallResponse = getPage("http://m.facebook.com/wall.php?id=".$fbWallId."&locale=en_US");
			preg_match('/name="post_form_id" value="(.*?)"/i', $wallResponse, $postFormId);

			if (isSet($postFormId[1])) {
				$postData = "post_form_id=".$postFormId[1]."&comment=".urlencode($statusFacebook)."&post=Post&locale=en_US";
				$statusFormUrl = "http://m.facebook.com/wall.php?id=".$fbWallId."&locale=en_US";

				// no way to check if this has gone right, FB do not return any message
				$updateResponse = getPage($statusFormUrl, $postData);
				fbDebug("Wall ".$fbWallId." update", $updateResponse, $fbDebug);

				$fbLogMessage .= "The status \"".$statusFacebook."\" has been published to the wall id ".$fbWallId."\n\n";
			}
		}

		// if wallId2, try post to the wanted wall
		if ($fbWallId2 != NULL && $fbWallId2 != "") {
			$wallResponse2 = getPage("http://m.facebook.com/wall.php?id=".$fbWallId2."&locale=en_US");
			preg_match('/name="post_form_id" value="(.*?)"/i', $wallResponse2, $postFormId2);

			if (isSet($postFormId2[1])) {
				$postData = "post_form_id=".$postFormId2[1]."&comment=".urlencode($statusFacebook)."&post=Post&locale=en_US";
				$statusFormUrl2 = "http://m.facebook.com/wall.php?id=".$fbWallId2."&locale=en_US";

				// no way to check if this has gone right, FB do not return any message
				$updateResponse2 = getPage($statusFormUrl2, $postData);
				fbDebug("Wall ".$fbWallId2." update", $updateResponse2, $fbDebug);

				$fbLogMessage .= "The status \"".$statusFacebook."\" has been published to the wall id ".$fbWallId2."\n\n";
			}
		}

		// logout from facebook so that a malicious plugin sniffing and sending FB session data won't be able to use it because it's already expired
		getPage("http://m.facebook.com/logout.php");
	}

	//Twitter?
	if ($twUser != null && $twPassword != null && $statusTwitter !== null) {

		$twUrl = 'http://twitter.com/statuses/update.xml';

		$postData = "status=".urlencode($statusTwitter);

		$twResponse = getPage($twUrl, $postData, $twUser, $twPassword);

		if ($twResponse === false || trim($twResponse) == "") {
			$fbLogMessage .= "The Twitter update didn't go that well :(.\n\n";
		} else {
			$fbLogMessage .= "The status \"".$statusTwitter."\" has been published to Twitter.\n\n";
		}
	}

	// everything has been done, clean the fbSessionData.txt file so that if someone tries to download it, it's empty
	deleteFbSessionData();

	// update the list of sent-to-facebook post ids

	$statusUpdate["fb-post-ids"] .= $post_ID."#";
	update_option('fb-status-update', $statusUpdate);

	$time = getMicrotime() - $startTime;

	$fbLogMessage .= "\nThe whole process took ".number_format($time, 3, ".", "")." seconds.\n";

	if ($fbDebug) {
		$fbLogMessage .= "\n\nThe following html is the response facebook gave to the plugin. If you can read html try discovering why facebook don't let the plugin login and update your status with user and password you provided. If you can't, please forward this email (JUST ONCE) to fcastaldo@gmail.com. The log does not contain your login credential but might contain personal data (friends update, last status etc.)\n\n";

		if ($fbDebugLog == null) {
			$fbLogMessage .= "The log is null. Odd";
		} else {
			foreach ($fbDebugLog as $action => $page) {
				$fbLogMessage .= "##############".$action."##############\n".$page."\n----------------------------------------------------------------\n\n\n";
			}
		}
	}	

	if ($fbLogEmail != null) {
		sendLogEmail($fbLogEmail, $fbLogMessage);
	}

	//exit(); //debug
}

$fbDebugLog = null;
function fbDebug($action, $page, $fbDebug) {
	global $fbDebugLog;
	if ($fbDebug) {
		if ($fbDebugLog == null) {
			$fbDebugLog = array();
		}
		$page = str_replace("\r\n", "", $page);
		$page = str_replace("\n", "", $page);
		$fbDebugLog[$action] = $page;
	}
}
?>